#
# Quick hack to enable gnome users to shutdown or reboot
# their workstation.
#
# This file contains shell functions to be used in gdm
# Session and PostSession scripts.
#
# Thomas Morin <t0m@free.fr>

GDMCONF=/etc/gdm/gdm.conf
DEFAULT_FILE=/etc/default/gdm

SHUTDOWN_FROM_SESSION_DEBUG_ONLY=0
SHUTDOWN_FROM_SESSION_ENABLED=1

ASKFILE_PREFIX="$HOME/.gdm-"

if [ -r $DEFAULT_FILE ]; then
  source $DEFAULT_FILE
fi


#
# Returns the value of a gdm.conf setting
#
function find_conf_value {
  KEY=$1
  grep -i "^$KEY *=" $GDMCONF | sed -e 's/^[^=]\+=//'
}


#
# Returns true if shutdown/reboot from session logout
# is allowed.
#
# For this to be true:
#   - SHUTDOWN_FROM_SESSION_ENABLED must be 1
#   - SystemMenu (in gdm.conf) must be true
#   - the gdm session has to be local
#
function is_shutdown_allowed {
  
  if [ $SHUTDOWN_FROM_SESSION_ENABLED -eq 0 ]; then
    return -1;
  fi
  
  # relevant for the postsession script,
  # this checks that gdm session is local
  #
  if [ -n "$REMOTE_HOST" ]; then
    return -1;
  fi

  if [ "`find_conf_value SecureSystemMenu`" == "true" ] ; then
    return -1;
  fi

  if [ "`find_conf_value SystemMenu`" == "true" ] ; then
    return 0;
  fi

  return -1
}

#
# returns the command defined in gdm.conf for shutdown and reboot
# (one per line)
# 
function get_commands_for { 
  find_conf_value "${1}Command" | tr ';' '\n'
}


#
# This function does the actual shutdown or reboot
# (it tries the ShutdownCommand or RebootCommand as configured 
# in gdm.conf)
#
function do_shutdown_command() {
   # parameter is whether reboot or shutdown
   command=$1

   get_commands_for $command | while read command; do
	   if [ "$SHUTDOWN_FROM_SESSION_DEBUG_ONLY" == 1 ]; then
		   logger gdm would try: $command
	   else
		   logger gdm trying : $command
		   eval $command
		   if [ $? -eq 0 ]; then
			   return 0
		   fi
	   fi
   done
}

#
# Check the existence of .dm-reboot or .dm-halt in the
# homedir of the logged out user.
# 
# If shutdown is allowed the desired action is made.
# 
# (used in gdm PostSession script)
#
function perhaps_shutdown_or_reboot {

	eval "HOMEDIR=~$USER"

        if is_shutdown_allowed ; then
          for asked in halt reboot; do
                if mv -f "${ASKFILE_PREFIX}${asked}" "${ASKFILE_PREFIX}${asked}.ongoing"; then
                   logger "GDM PostSession script : user $USER asked $asked !!"
                   do_shutdown_command $asked
                   rm -f ${ASKFILE_PREFIX}*
                fi
          done
        fi
}

